<?php
require_once('testcase_header.php');
require_once('toy_service.php');
require_once('base/DispatchService.php');
require_once('lib/url.php');
require_once('lib/Log.php');

class DispatchServiceTest extends UnitTestCase {

    function DispatchService() {
        $this->UnitTestCase();
    }

    private static function makeRequest($method_name, $path) {
        $r = new Request();
        $r->server["PATH_INFO"] = $path;
        $r->server["REQUEST_URI"] = $path;
        $r->server["REQUEST_METHOD"] = $method_name;
        return $r;
    }

    public function testDipatch_1() {
        $tsa = new ToyService("a");
        $tsb = new ToyService("b");

        $service = new base_DispatchService("", false, array(
            array("/a", $tsa),
            array("/b", $tsb) ));

        $r = $service->process_request(DispatchServiceTest::makeRequest("GET", "/"));
        $this->assertEqual($r->status_code, 404);

        $r = $service->process_request(DispatchServiceTest::makeRequest("GET", "/sadfasdf"));
        $this->assertEqual($r->status_code, 404);

        $service->process_request(DispatchServiceTest::makeRequest("GET", "/a"));
        $service->process_request(DispatchServiceTest::makeRequest("POST", "/b"));

        $this->assertEqual($tsa->request_history[0]->server["PATH_INFO"], "/a");
        $this->assertEqual($tsa->request_history[0]->server["REQUEST_METHOD"], "GET");

        $this->assertEqual($tsb->request_history[0]->server["PATH_INFO"], "/b");
        $this->assertEqual($tsb->request_history[0]->server["REQUEST_METHOD"], "POST");
    }

    public function testDipatch_2() {
        $tsa = new ToyService("a");
        $tsb = new ToyService("b");

        $service = new base_DispatchService("", false, array(
            array("/sas/a", $tsa),
            array("/sas/b", $tsb)) );

        $r = $service->process_request(DispatchServiceTest::makeRequest("GET", "/weee"));
        $this->assertEqual($r->status_code, 404);

        $r = $service->process_request(DispatchServiceTest::makeRequest("GET", "/sas/e"));
        $this->assertEqual($r->status_code, 404);

        $r = $service->process_request(DispatchServiceTest::makeRequest("GET", "/sasss"));
        $this->assertEqual($r->status_code, 404);


        $r = $service->process_request(DispatchServiceTest::makeRequest("GET", "/sas"));
        $this->assertEqual($r->status_code, 404);

        $service->process_request(DispatchServiceTest::makeRequest("GET", "/sas/a"));
        $service->process_request(DispatchServiceTest::makeRequest("POST", "/sas/b"));

        $this->assertEqual($tsa->request_history[0]->server["PATH_INFO"], "/sas/a");
        $this->assertEqual($tsa->request_history[0]->server["REQUEST_METHOD"], "GET");

        $this->assertEqual($tsb->request_history[0]->server["PATH_INFO"], "/sas/b");
        $this->assertEqual($tsb->request_history[0]->server["REQUEST_METHOD"], "POST");
    }

    public function testEmptySubpath() {
        $tsa = new ToyService("a");
        $tsb = new ToyService("b");
        $tsc = new ToyService("c");

        $service = new base_DispatchService("", false, array(
            array("some/c", $tsc),
            array("some/", $tsb),
            array("some", $tsa),
            ));

        $service->process_request(DispatchServiceTest::makeRequest("GET", "some"));
        $service->process_request(DispatchServiceTest::makeRequest("GET", "some/"));
        $service->process_request(DispatchServiceTest::makeRequest("GET", "some/c"));

        $this->assertEqual($tsa->request_history[0]->server["PATH_INFO"], "some");
        $this->assertEqual($tsa->request_history[0]->server["REQUEST_METHOD"], "GET");

        $this->assertEqual($tsb->request_history[0]->server["PATH_INFO"], "some/");
        $this->assertEqual($tsb->request_history[0]->server["REQUEST_METHOD"], "GET");

        $this->assertEqual($tsc->request_history[0]->server["PATH_INFO"], "some/c");
        $this->assertEqual($tsc->request_history[0]->server["REQUEST_METHOD"], "GET");
    }

    public function testWildcards() {
        $tsa = new ToyService("a");
        $tsb = new ToyService("b");
        $tsc = new ToyService("c");

        $service = new base_DispatchService("", false, array(
            array("/stubs/((log_id))/gems", $tsb),
            array("/stones/((ruby))/wei((weight))", $tsa),
            array("/stones/((ruby))/((weight))", $tsa),
            array("((root))/c", $tsc),
            ));

        $service->process_request(DispatchServiceTest::makeRequest('GET', '/stubs/ironbat/gems'));
        $service->process_request(DispatchServiceTest::makeRequest('GET', '/stones/eee/241'));
        $service->process_request(DispatchServiceTest::makeRequest('GET', '/stones/yo/weight'));
        $service->process_request(DispatchServiceTest::makeRequest('GET', 'some/c?mode=hinder'));

        $this->assertEqual($tsb->request_history[0]->server['PATH_INFO'], '/stubs/ironbat/gems');
        $this->assertEqual($tsb->request_history[0]->server['REQUEST_METHOD'], 'GET');
        $this->assertEqual($tsb->request_history[0]->get_args['log_id'], 'ironbat');

        $this->assertEqual($tsa->request_history[0]->server['PATH_INFO'], '/stones/eee/241');
        $this->assertEqual($tsa->request_history[0]->server['REQUEST_METHOD'], "GET");
        $this->assertEqual($tsa->request_history[0]->get_args['ruby'], 'eee');
        $this->assertEqual($tsa->request_history[0]->get_args['weight'], '241');

        $this->assertEqual($tsa->request_history[1]->server['PATH_INFO'], '/stones/yo/weight');
        $this->assertEqual($tsa->request_history[1]->server['REQUEST_METHOD'], "GET");
        $this->assertEqual($tsa->request_history[1]->get_args['ruby'], 'yo');
        $this->assertEqual($tsa->request_history[1]->get_args['weight'], 'ght');

        $this->assertEqual($tsc->request_history[0]->server['PATH_INFO'], 'some/c?mode=hinder');
        $this->assertEqual($tsc->request_history[0]->server['REQUEST_METHOD'], 'GET');
        $this->assertEqual($tsc->request_history[0]->get_args['root'], 'some');
   }

   public function testPrefix() {
       // Test that DispatchService's prefix works as expected. (Should be prepended to path patterns)
        $tsa = new ToyService("a");
        $tsb = new ToyService("b");

        $prefix = 'some/prefix';
        $service = new base_DispatchService($prefix, false, array(
            array("/stub", $tsa),
            array("/ton", $tsb),
            ));

        $service->process_request(DispatchServiceTest::makeRequest('GET', $prefix . '/stub'));
        $service->process_request(DispatchServiceTest::makeRequest('GET', $prefix . '/ton'));

        $this->assertEqual($tsa->request_history[0]->server['PATH_INFO'], $prefix . '/stub');
        $this->assertEqual($tsb->request_history[0]->server['PATH_INFO'], $prefix . '/ton');
   }

   public function testWildcards2() {
        $tsa = new ToyService("a");
        $tsb = new ToyService("b");

        $service = new base_DispatchService("", false, array(
            array("/sas/((a))/props", $tsa),
            array("/sas/((a))", $tsb)) );

        $service->process_request(DispatchServiceTest::makeRequest("GET", "/sas/something"));
        $service->process_request(DispatchServiceTest::makeRequest("GET", "/sas/eee/props"));
        $service->process_request(DispatchServiceTest::makeRequest("GET", "/sp/accounts/asdfasdfasdfasdf/props"));

        $this->assertEqual($tsa->request_history[0]->server["PATH_INFO"], "/sas/eee/props");
        $this->assertEqual($tsb->request_history[0]->server["PATH_INFO"], "/sas/something");
    }

}


// $t = new DispatchServiceTest();
// $t->run(new TextReporter());


?>